#!/bin/python3
#%%
import pandas as pd
from tools.std_lib.basic import find_matching_paths, json_load
from tools.data_processing.easy_pandas import import_py_module

#%%


#%%
dir_path = './cache/clf'
json_paths = find_matching_paths(dir_path, pattern=r'info\.json')

#%%

def load_json_and_add_save_path(path):
    d = json_load(path=path)
    d['save_path'] = path
    d['xgb_params'] = str(d['clf']['my_params'])
    return d

info_dict_list = [load_json_and_add_save_path(path) for path in json_paths]
info_dict_s = pd.Series(info_dict_list)
info_df = pd.json_normalize(info_dict_s)
info_df.index = pd.Index(range(1, len(info_df)+1), name='index')
info_df

#%%
info_df['n_sample'] = info_df['main.X_df.shape'].apply(lambda s: s[0])
info_df['n_features'] = info_df['main.X_df.shape'].apply(lambda s: s[1])

#%%

def get_drop_cols(info_df: pd.DataFrame):
    drop_cols = pd.Series(False, index = info_df.columns)
    drop_cols |= info_df.columns.str.startswith('main.y_df.shape')
    drop_cols |= info_df.columns.str.startswith('main.X_df.shape')
    drop_cols |= info_df.columns.str.startswith('feature.fillna')
    drop_cols |= info_df.columns.str.startswith('feature.features')
    drop_cols |= info_df.columns.str.startswith('feature.feature_importances_info.feature_importances')
    drop_cols |= info_df.columns.str.startswith('clf.')

    drop_cols |= info_df.columns.str.endswith('bins_cnt_10')
    drop_cols |= info_df.columns.str.endswith('bins_cnt_20')
    drop_cols |= info_df.columns.str.endswith('lift_01')
    drop_cols |= info_df.columns.str.endswith('lift_03')
    drop_cols |= info_df.columns.str.endswith('lift_04')
    drop_cols |= info_df.columns.str.endswith('lift_10')
    drop_cols |= info_df.columns.str.endswith('lift_20')
    drop_cols |= info_df.columns.str.endswith('lift_30')
    drop_cols |= info_df.columns.str.endswith('lift_40')
    drop_cols |= info_df.columns.str.endswith('lift_50')

    return drop_cols

drop_cols = get_drop_cols(info_df)
info_df.loc[:, ~drop_cols].columns.to_list()

#%%
info_df.loc[:, ~drop_cols]

# %%
info_df = info_df.loc[:, ~drop_cols].copy()

#%%
info_df.columns = [c.split('.')[-1] for c in info_df.columns]

#%%
import datetime
info_df['timestamp'] = info_df['timestamp'].apply(lambda s: datetime.datetime.strptime(s, '%Y%m%d%a%H%M%S'))

#%%
info_df['abs_overfit_auc'] = info_df['train_auc'] - info_df['validate_auc']
info_df['abs_overfit_ks'] = info_df['train_ks'] - info_df['validate_ks']

info_df['abs_decline_auc'] = info_df['validate_auc'] - info_df['test_auc']
info_df['abs_decline_ks'] = info_df['validate_ks'] - info_df['test_ks']

info_df['abs_overfit'] = info_df['abs_overfit_auc'].abs() + info_df['abs_overfit_ks'].abs()
info_df['abs_decline'] = info_df['abs_decline_auc'].abs() + info_df['abs_decline_ks'].abs()

info_df['abs_gap'] = info_df['abs_overfit'].abs() + info_df['abs_decline'].abs()

info_df['select'] = float('nan')

#%%
sorted_cols = [
    'prefix',

    'exp_name',
    'save_name',
    'save_path',
    'uuid_hex',

    'timestamp',

    'classifier_name',
    'xgb_params',

    'n_sample',
    'n_features',
    'n_features_in_use',

    'train_auc',
    'validate_auc',
    'test_auc',

    'train_ks',
    'validate_ks',
    'test_ks',
    
    'abs_overfit_auc',
    'abs_overfit_ks',
    'abs_decline_auc',
    'abs_decline_ks',
    'abs_overfit',
    'abs_decline',
    'abs_gap',

    '1970-01_auc',
    '1970-01_ks',

    'train-lift_02',
    'validate-lift_02',
    'test-lift_02',
    'train-lift_05',
    'validate-lift_05',
    'test-lift_05',

    
    'train-bins_cnt_50',
    'validate-bins_cnt_50',
    'test-bins_cnt_50',
    
    ]

info_df = info_df[sorted_cols].copy()

info_df.sort_values(['prefix', 'test_ks'], ascending=[True, False], inplace=True)

#%%
import os
from tools.std_lib.basic import get_timestamp
from tools.tables.colorful_excel import ExcelModifier

excel_modifier = ExcelModifier(excel_data=info_df, regex=True)
excel_modifier.set_column_dimensions(force=True)
excel_modifier.set_sheet_filter()
excel_modifier.set_column_property(auto=True)

hidden_cols = [
    'exp_name',
    'save_name',
    'save_path',
    'uuid_hex',

    'classifier_name',
]
excel_modifier[:][:][hidden_cols].set_column_hidden()

excel_modifier[:][:][r'(?:train|validate|test)_auc'].set_column_conditional_format(rule='ColorScale', solo=False)
excel_modifier[:][:][r'(?:train|validate|test)_ks'].set_column_conditional_format(rule='ColorScale', solo=False)

excel_modifier[:][:][r'abs_overfit|abs_decline|abs_gap'].set_column_conditional_format(rule='ColorScale', solo=False)
excel_modifier[:][:][r'lift'].set_column_conditional_format(rule='ColorScale', solo=False)

bold_cols = [
    'abs_overfit',
    'abs_decline',
    'abs_gap',

    'test_auc', 'test_ks',
]
excel_modifier[:][:][bold_cols].set_column_property(fonts = {'bold': True})

ts = get_timestamp()
xlsx_name = f'{upper_batch}-src-perf-stat-{ts}.xlsx'
xlsx_path = os.path.join(modeling_path, 'stat/perf', xlsx_name)

excel_modifier.save(f'./tmp.xlsx')

#%%


